
import logging
import time
from pathlib import PosixPath, Path
import requests
from . import cgi
from .cgi import URL, DEFAULT_DIR


logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


def by_new_arrivals(*filters, remote_dir=DEFAULT_DIR, dest="."):
    """Monitor `remote_dir` on FlashAir card for new files.
    When new files are found, should they pass all of the given
    `filters`, sync them with `dest` local directory."""
    old_files = set()
    while True:
        new_files = set(cgi.list_files(*filters, remote_dir=remote_dir))
        if old_files and old_files < new_files:
            new_arrivals = new_files - old_files
            logger.info("Files to sync:\n{}".format(
                "\n".join("  " + f.filename for f in new_arrivals)))
            for f in new_arrivals:
                _sync_file(dest, f)
        old_files = new_files
        time.sleep(1)


def by_files(to_sync, dest="."):
    """Sync a given list of files from `cgi.list_files` to `dest` dir"""
    for f in to_sync:
        _sync_file(dest, f)


def by_time(*filters, remote_dir=DEFAULT_DIR, dest=".", count=1):
    """Sync most recent file by date, time attribues"""
    files = cgi.list_files(*filters, remote_dir=remote_dir)
    most_recent = sorted(files, key=lambda f: (f.date, f.time))
    to_sync = most_recent[-count:]
    logger.info("Files to sync:\n{}".format(
        "\n".join("  " + f.filename for f in to_sync)))
    for f in to_sync[::-1]:
        _sync_file(dest, f)


def by_name(*filters, remote_dir=DEFAULT_DIR, dest=".", count=1):
    """Sync files whose filename attribute is highest in alphanumeric order"""
    files = cgi.list_files(*filters, remote_dir=remote_dir)
    greatest = sorted(files, key=lambda f: f.filename)
    to_sync = greatest[-count:]
    logger.info("Files to sync:\n{}".format(
        "\n".join("  " + f.filename for f in to_sync)))
    for f in to_sync[::-1]:
        _sync_file(dest, f)


def _sync_file(destination_dir, fileinfo):
    local_path = Path(destination_dir, fileinfo.filename)
    if local_path.exists():
        logger.info("File '{}' already exists; not syncing from SD card".format(
                    str(local_path)))
    else:
        remote_path = PosixPath(fileinfo.directory, fileinfo.filename)
        logger.info("Copying remote file {} to {}".format(
                    str(remote_path), str(local_path)))
        streaming_file = _get_file(fileinfo)
        _write_file(str(local_path), streaming_file)


def _get_file(fileinfo):
    path = PosixPath(fileinfo.directory, fileinfo.filename)
    flashair_url = (URL + str(path)).encode("UTF-8")
    logger.debug("Requesting file: {}".format(flashair_url)) 
    request = requests.get(flashair_url, stream=True)
    return request


def _write_file(local_path, response):
    if response.status_code == 200:
        with open(local_path, "wb") as outfile:
            for chunk in response.iter_content(10**6):
                outfile.write(chunk)
    else:
        raise requests.RequestException("Expected status code 200")




import logging
import requests
from collections import namedtuple
from enum import Enum


logger = logging.getLogger(__name__)
logging.getLogger("requests").setLevel(logging.WARNING)


##################################
# FlashAir API request formatting


URL = "http://flashair"
DEFAULT_DIR = "/DCIM/100__TSB"


class Op(Enum):
    list_files = 100
    count_files = 101


def make_cmd(op, **params):
    extras = _pairs(params) if params else ""
    cmd = "{url}/command.cgi?op={op:d}{extras}".format(
        url=URL, op=op.value, extras=extras)
    return cmd.encode("UTF-8")


def _pairs(keyvals):
    pairs = "&".join("{}={}".format(key, val)
                     for key, val in keyvals.items())
    return "&" + pairs


##########################
# FlashAir API functions


def list_files(*filters, remote_dir=DEFAULT_DIR):
    response = _cgi_cmd(Op.list_files, DIR=remote_dir)
    files = _split_file_list(response.text)
    return (f for f in files if all(filt(f) for filt in filters))


def count_files(remote_dir=DEFAULT_DIR):
    response = _cgi_cmd(Op.count_files, DIR=remote_dir)
    return int(response.text)


#############################
# API implementation details


def _cgi_cmd(op, **extras):
    cmd = make_cmd(op, **extras)
    logger.debug("Request: {}".format(cmd))
    response = requests.get(cmd)
    logger.debug("Response: {}".format(response))
    return response


_fields = ["directory", "filename", "size", "attribute", "date", "time"]
FileInfo = namedtuple("FileInfo", _fields)


def _split_file_list(text):
    lines = text.split("\r\n")
    for line in lines:
        groups = line.split(",")
        if len(groups) == 6:
            d, f, size, a, date, time = groups
            yield FileInfo(d, f, int(size), a, int(date), int(time))


if __name__ == "__main__":
    print(list_files())
    print(count_files())




from setuptools import setup
import tfatool._version


setup(name="tfatool",
      version=tfatool._version.__version__,
      scripts=["flashair-util"],
      install_requires=["requests>=2.9.0",],
      licence="MIT",
      packages=["tfatool"],
      description=("Tools for syncing files with the "
                   "Toshiba FlashAir wireless SD card"),
      author="Tad Leonard",
      author_email="tadfleonard@gmail.com",
      keywords="wireless sd card sync toshiba flashair", 
      url="https://github.com/TadLeonard/tfatool",
      download_url="https://github.com/TadLeonard/tfatool/archive/v1.1.0.tar.gz",
)

